/**********************************************************************
* project name:  BogoMIPS
* file name:     bogomips.c
* initial date:  8/05/2002
* author:        Francesco Orabona (bremen79@genie.it)
* description:   BogoMIPS calculator for TI-89/92+
* version:       1.00
*
* Based on the Linux kernel code in init/main.c and __delay as defined
* on m68k platform.
* Also based on Jeff Tranter's bogomips calculator.
*
* For more information see the BogoMIPS Mini-HOWTO document.
*
* NOTE: AMS 2.0x VERSION
**********************************************************************/

#define OPTIMIZE_ROM_CALLS
#define SAVE_SCREEN
#define NO_EXIT_SUPPORT
#define NO_CALC_DETECT
#define MIN_AMS 200

#define USE_TI89
#define USE_TI92P

#include <kbd.h>
#include <stdio.h>
#include <graph.h>
// FiftyMsecTick has a wrong definition in TIGCC 0.94 beta 13 and ealier version
#define FiftyMsecTick (*((volatile unsigned long*)(_rom_call_addr(4FC))))

#define HZ 20

extern __inline__ void __delay(unsigned long loops)
{
	__asm__ __volatile__ ("1: subql #1,%0; jcc 1b"
		: "=d" (loops) : "0" (loops));
}


/* This is the number of bits of precision for the loops_per_jiffy.  Each
   bit takes on average 1.5/HZ seconds.  This (like the original) is a little
   better than 1% */
#define LPS_PREC 8

void _main(void)
{
	unsigned long ticks, loopbit;
	int lps_precision = LPS_PREC;

  // this should be approx 2 Bo*oMips to start (note initial shift), and will
  // still work even if initially too large, it will just take slightly longer

	unsigned long loops_per_jiffy = (1<<12);

	ClrScr();
	DrawStr(0,0,"Calibrating delay loop...",A_NORMAL);
	while (loops_per_jiffy <<= 1) {
		// wait for "start of" clock tick 
		ticks = FiftyMsecTick;
		while (ticks == FiftyMsecTick)
			; // nothing
		// Go .. 
		ticks = FiftyMsecTick;
		__delay(loops_per_jiffy);
		ticks = FiftyMsecTick - ticks;
		if (ticks)
			break;
	}

  // Do a binary approximation to get loops_per_jiffy set to equal one clock
  // (up to lps_precision bits) 
	loops_per_jiffy >>= 1;
	loopbit = loops_per_jiffy;
	while ( lps_precision-- && (loopbit >>= 1) ) {
		loops_per_jiffy |= loopbit;
		ticks = FiftyMsecTick;
		while (ticks == FiftyMsecTick);
		ticks = FiftyMsecTick;
		__delay(loops_per_jiffy);
		if (FiftyMsecTick != ticks)	// longer than 1 tick 
			loops_per_jiffy &= ~loopbit;
	}

  // Round the value and print it 
	printf_xy(0,10,"%lu.%02lu BogoMIPS",
		loops_per_jiffy/(500000/HZ),
		(loops_per_jiffy/(5000/HZ)) % 100);
	ngetchx();
}